/**
 *
 */
package edu.agh.sm2006.smschess.server;

import static org.easymock.EasyMock.*;

import java.io.IOException;
import java.util.Properties;

import junit.framework.TestCase;
import edu.agh.sm2006.smschess.server.cimd.CIMDParameterType;
import edu.agh.sm2006.smschess.server.cimd.DeliverMessageConsumer;
import edu.agh.sm2006.smschess.server.cimd.message.Message;
import edu.agh.sm2006.smschess.server.cimd.message.MessageType;
import edu.agh.sm2006.smschess.server.config.XBoardEngineConfiguration;
import edu.agh.sm2006.smschess.server.message.MessageParser;
import edu.agh.sm2006.smschess.server.message.MessageParserImpl;
import edu.agh.sm2006.smschess.server.session.HumanSession;
import edu.agh.sm2006.smschess.server.session.SessionManager;
import edu.agh.sm2006.smschess.server.session.XBoardSession;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcessManager;

/**
 * @author Kuba Madej
 * 
 */
public class IntegrationTest extends TestCase {
	private static final int XBOARD_ENGINE_TIMEOUT;

	private static final String CHESS_ENGINE;

	static {
		final Properties props = new Properties();
		try {
			props.load(IntegrationTest.class.getClassLoader()
					.getResourceAsStream("xboard.properties"));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		CHESS_ENGINE = props.getProperty("engine");
		XBOARD_ENGINE_TIMEOUT = Integer.parseInt(props.getProperty("timeout"));
	}

	Connector connector = createMock(Connector.class);

	private XBoardEngineProcessManager engineProcessManager;

	private SessionManager sessionManager;

	private MessageParser chessMessageParser;

	private DeliverMessageConsumer deliverMessageConsumer;

	@Override
	protected void setUp() throws Exception {
		engineProcessManager = new XBoardEngineProcessManager(
				new XBoardEngineConfiguration() {

					public int getMaxProcesses() {
						return 10;
					}

					public String getXBoardEngineStartCommand() {
						return CHESS_ENGINE;
					}

				});
		sessionManager = new SessionManager(engineProcessManager, connector);
		chessMessageParser = new MessageParserImpl(sessionManager);
		deliverMessageConsumer = new DeliverMessageConsumer(connector,
				chessMessageParser);
	}

	@Override
	protected void tearDown() throws Exception {
		for (XBoardSession session : sessionManager.getXBoardSessions()
				.values()) {
			session.terminate(false);
		}
	}

	public void testConsumeCreatePrivateHumanSessionMessage() throws Exception {
		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "c:15x0:");

		connector.sendMessage(eq("505"), eq("s"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertEquals(1, sessionManager.getHumanSessions().size());
		assertNotNull(sessionManager.getHumanSessions().get("15x0"));
		assertFalse(sessionManager.getHumanSessions().get("15x0").isPublic());

		verify(connector);
	}

	public void testConsumeCreatePublicHumanSessionMessage() throws Exception {
		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "c:15x0:p");

		connector.sendMessage(eq("505"), eq("s"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertEquals(1, sessionManager.getHumanSessions().size());
		assertNotNull(sessionManager.getHumanSessions().get("15x0"));
		assertTrue(sessionManager.getHumanSessions().get("15x0").isPublic());

		verify(connector);
	}

	public void testConsumeCreateXBoardSessionMessage() throws Exception {
		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "c::x");

		connector.sendMessage(eq("505"), eq("x:" + (Long.MIN_VALUE)));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertEquals(0, sessionManager.getHumanSessions().size());
		assertEquals(1, sessionManager.getXBoardSessions().size());
		assertNotNull(sessionManager.getXBoardSessions().get(
				"" + (Long.MIN_VALUE)));

		verify(connector);
	}

	public void testConsumeCreateHumanSessionMessageInvalidSessionId()
			throws Exception {
		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "c:15x0:");

		connector.sendMessage(eq("505"), eq("s"));
		connector.sendMessage(eq("505"), eq("e:isid"));
		replay(connector);

		deliverMessageConsumer.consume(message);
		deliverMessageConsumer.consume(message);

		assertEquals(1, sessionManager.getHumanSessions().size());
		assertEquals(0, sessionManager.getXBoardSessions().size());

		verify(connector);
	}

	public void testConsumeCreateXBoardSessionMessageException()
			throws Exception {
		engineProcessManager = new XBoardEngineProcessManager(
				new XBoardEngineConfiguration() {

					public int getMaxProcesses() {
						return 1;
					}

					public String getXBoardEngineStartCommand() {
						return CHESS_ENGINE;
					}

				});
		sessionManager = new SessionManager(engineProcessManager, connector);
		chessMessageParser = new MessageParserImpl(sessionManager);
		deliverMessageConsumer = new DeliverMessageConsumer(connector,
				chessMessageParser);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "c::x");

		connector.sendMessage(eq("505"), eq("x:" + (Long.MIN_VALUE)));
		connector.sendMessage(eq("505"), eq("e:xb"));
		replay(connector);

		deliverMessageConsumer.consume(message);
		deliverMessageConsumer.consume(message);

		assertEquals(0, sessionManager.getHumanSessions().size());
		assertEquals(1, sessionManager.getXBoardSessions().size());

		verify(connector);
	}

	public void testConsumeJoinPrivateSessionMessage() throws Exception {
		HumanSession session = new HumanSession("15x0", sessionManager,
				connector, "602", false);
		sessionManager.getHumanSessions().put("15x0", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "j:15x0");

		connector.sendMessage(eq("602"), eq("g"));
		connector.sendMessage(eq("505"), eq("s"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNotNull(sessionManager.getHumanSessions().get("15x0")
				.getSecondaryPlayerId());

		verify(connector);
	}

	public void testConsumeJoinPublicSessionMessage() throws Exception {
		HumanSession session = new HumanSession("15x0", sessionManager,
				connector, "602", true);
		sessionManager.getHumanSessions().put("15x0", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "j:");

		connector.sendMessage(eq("602"), eq("g"));
		connector.sendMessage(eq("505"), eq("x:15x0"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNotNull(sessionManager.getHumanSessions().get("15x0")
				.getSecondaryPlayerId());

		verify(connector);
	}

	public void testConsumeJoinPublicSessionMessageNotFound() throws Exception {
		HumanSession session = new HumanSession("15x0", sessionManager,
				connector, "602", false);
		sessionManager.getHumanSessions().put("15x0", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "j:");

		connector.sendMessage(eq("505"), eq("e:nopublic"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNull(sessionManager.getHumanSessions().get("15x0")
				.getSecondaryPlayerId());

		verify(connector);
	}

	public void testConsumeJoinPrivateSessionMessageNotFound() throws Exception {
		HumanSession session = new HumanSession("15x", sessionManager,
				connector, "602", false);
		sessionManager.getHumanSessions().put("15x", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "j:15x0");

		connector.sendMessage(eq("505"), eq("e:isid"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNull(sessionManager.getHumanSessions().get("15x")
				.getSecondaryPlayerId());

		verify(connector);
	}

	public void testConsumeQuitMessageHumanSession() throws Exception {
		HumanSession session = new HumanSession("15x", sessionManager,
				connector, "602", false);
		session.join("602");
		sessionManager.getHumanSessions().put("15x", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "q:15x");

		connector.sendMessage(eq("602"), eq("q"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNull(sessionManager.getHumanSessions().get("15x"));

		verify(connector);
	}

	public void testConsumeQuitMessageXBoardSession() throws Exception {
		XBoardSession session = new XBoardSession("15x", sessionManager,
				engineProcessManager, connector, "502");
		sessionManager.getXBoardSessions().put("15x", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "q:15x");

		replay(connector);

		deliverMessageConsumer.consume(message);

		assertNull(sessionManager.getXBoardSessions().get("15x"));

		verify(connector);
	}

	public void testConsumeMoveMessageHumanSessionWhite() throws Exception {
		HumanSession session = new HumanSession("15x", sessionManager,
				connector, "502", false);
		session.join("602");
		sessionManager.getHumanSessions().put("15x", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "505");
		message.addParameter(CIMDParameterType.UserData, "m:15x:a2a3");

		connector.sendMessage(eq("505"), eq("s"));
		connector.sendMessage(eq("602"), eq("m:a2a3"));
		replay(connector);

		deliverMessageConsumer.consume(message);

		verify(connector);
	}

	public void testConsumeMoveMessageHumanSessionBlack() throws Exception {
		HumanSession session = new HumanSession("15x", sessionManager,
				connector, "502", false);
		sessionManager.getHumanSessions().put("15x", session);
		session.join("602");

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "602");
		message.addParameter(CIMDParameterType.UserData, "m:15x:a7a5");

		connector.sendMessage(eq("602"), eq("m:a2a3"));
		connector.sendMessage(eq("602"), eq("s"));
		connector.sendMessage(eq("502"), eq("m:a7a5"));
		replay(connector);

		session.move("a2a3");

		deliverMessageConsumer.consume(message);

		verify(connector);
	}

	public void testConsumeMoveMessageXBoardSession() throws Exception {
		XBoardSession session = new XBoardSession("15x", sessionManager,
				engineProcessManager, connector, "502");
		sessionManager.getXBoardSessions().put("15x", session);

		Message message = new Message(MessageType.DeliverMessage);
		message.addParameter(CIMDParameterType.OriginatorAddress, "502");
		message.addParameter(CIMDParameterType.UserData, "m:15x:a2a3");

		connector.sendMessage(eq("502"), eq("s"));
		connector.sendMessage((String) anyObject(), (String) anyObject());
		replay(connector);

		deliverMessageConsumer.consume(message);

		Thread.sleep(XBOARD_ENGINE_TIMEOUT);

		verify(connector);
	}
}
