/**
 *
 */
package edu.agh.sm2006.smschess.server.cimd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import edu.agh.sm2006.smschess.server.Connector;
import edu.agh.sm2006.smschess.server.cimd.message.LoginRequestMessage;
import edu.agh.sm2006.smschess.server.cimd.message.LogoutRequestMessage;
import edu.agh.sm2006.smschess.server.cimd.message.Message;
import edu.agh.sm2006.smschess.server.cimd.message.MessageType;
import edu.agh.sm2006.smschess.server.cimd.message.SubmitRequestMessage;
import edu.agh.sm2006.smschess.server.config.CIMDConfiguration;
import edu.agh.sm2006.smschess.server.config.Configuration;
import edu.agh.sm2006.smschess.server.message.MessageParser;

/**
 * @author Kuba Madej
 * 
 */
public class CIMDConnector implements Connector {
	public static final String ENCODING = "US-ASCII";

	public final static char STX = 2;

	public final static char TAB = 9;

	public final static char ETX = 3;

	private static final char DELIMITER = ':';

	private static final Logger logger = Logger.getLogger(CIMDConnector.class);

	private int packetNumber;

	private CIMDConfiguration configuration;

	private OutputStreamWriter outputStreamWriter;

	private Map<MessageType, CIMDMessageConsumer> consumers = new HashMap<MessageType, CIMDMessageConsumer>();

	private InputStream inputStream;

	private final String addressString;

	private final int port;

	private Socket socket;

	public CIMDConnector(Configuration configuration) {
		this.configuration = configuration;
		addressString = configuration.getCIMDHost();
		port = configuration.getCIMDPort();

		connect();

		Runtime.getRuntime().addShutdownHook(new Thread() {
			@Override
			public void run() {
				sendMessage(new LogoutRequestMessage());
			}
		});
	}

	/**
	 * Connects or reconnects server to the SMSC service.
	 * 
	 */
	public void connect() {
		do {
			logger.info("Connecting to " + addressString + ":" + port);

			try {
				socket.close();
			} catch (Exception e) {
				// ignore, it may be not connected or even null
			}

			InetAddress address;
			try {
				address = InetAddress.getByName(addressString);
			} catch (UnknownHostException e1) {
				logger.error(e1);
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					// nothing to do
				}
				continue;
			}

			try {
				socket = new Socket(address, port);

				inputStream = socket.getInputStream();
				outputStreamWriter = new OutputStreamWriter(socket
						.getOutputStream(), ENCODING);
				packetNumber = 1;
			} catch (IOException e) {
				logger.error(e);
				continue;
			}
			break;
		} while (true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDConnectorI#openConnection(edu.agh.sm2006.smschess.server.message.SMSChessMessageParser)
	 */
	public void openConnection(MessageParser chessMessageParser) {
		login();

		DeliverMessageConsumer deliverMessageConsumer = new DeliverMessageConsumer(
				this, chessMessageParser);
		consumers.put(MessageType.DeliverMessage, deliverMessageConsumer);
	}

	private String getNextPacketNumber() {
		final String packetNumber = String.format("%03d", this.packetNumber);
		this.packetNumber += 2;
		if (this.packetNumber > 255)
			this.packetNumber = 1;
		return packetNumber;
	}

	void login() {
		logger.info("Logging in");
		consumers.put(MessageType.LoginResponse, new CIMDMessageConsumer() {

			public void consume(Message message) {
				for (CIMDParameter parameter : message.getParameters()) {
					if (parameter.getType() == CIMDParameterType.ErrorCode) {
						logger.error("Login failed");
						System.exit(-1);
					}
				}
				logger.info("Logged in succesfully");
				consumers.remove(MessageType.LoginResponse);
			}

		});

		sendMessage(new LoginRequestMessage(configuration.getCIMDUserName(),
				configuration.getCIMDPassword()));
	}

	public void sendMessage(String receiver, String messageContent) {
		sendMessage(new SubmitRequestMessage(receiver, messageContent));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDConnectorI#sendMessage(edu.agh.sm2006.smschess.server.cimd.message.Message)
	 */
	public void sendMessage(final Message message) {
		sendMessage(message, getNextPacketNumber());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDConnectorI#sendMessage(edu.agh.sm2006.smschess.server.cimd.message.Message,
	 *      java.lang.String)
	 */
	public synchronized void sendMessage(final Message message,
			final String messageNumber) {
		logger.debug("Sending message: " + message);
		try {
			outputStreamWriter.append(STX);
			outputStreamWriter.append(message.getType().toString());
			outputStreamWriter.append(DELIMITER);
			outputStreamWriter.append(messageNumber);
			outputStreamWriter.append(TAB);
			for (CIMDParameter parameter : message.getParameters()) {
				outputStreamWriter.append(parameter.getType().toString());
				outputStreamWriter.append(DELIMITER);
				outputStreamWriter.append(parameter.getValue());
				outputStreamWriter.append(TAB);
			}
			outputStreamWriter.append(ETX);
			outputStreamWriter.flush();
		} catch (IOException e) {
			logger.error(e);
			connect();
			sendMessage(message);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDConnectorI#messageReceived(edu.agh.sm2006.smschess.server.cimd.message.Message)
	 */
	public void messageReceived(Message message) {
		logger.debug("Received message: " + message);
		CIMDMessageConsumer consumer = consumers.get(message.getType());
		if (consumer != null)
			consumer.consume(message);
		else
			logger.debug("Consumer not found for message type "
					+ message.getType());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.agh.sm2006.smschess.server.cimd.CIMDConnectorI#getInputStream()
	 */
	public InputStream getInputStream() {
		return inputStream;
	}

	public void runReceiver() {
		new CIMDMessageReceiver(this).run();
	}

}
