/**
 *
 */
package edu.agh.sm2006.smschess.server.session;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;

import edu.agh.sm2006.smschess.server.Connector;
import edu.agh.sm2006.smschess.server.message.server2client.StartMessage;
import edu.agh.sm2006.smschess.server.session.xboard.XBoardEngineProcessManager;

/**
 * @author Kuba Madej
 *
 */
public class SessionManager {
	private static final long HALF_DAY = 1000 * 60 * 60 * 12;

	private long nextXBoardSessionId = Long.MIN_VALUE;

	/**
	 * Map holding session identifiers chosen by players and coresponding
	 * sessions
	 */
	private Map<String, XBoardSession> xBoardSessions = new HashMap<String, XBoardSession>();

	private Map<String, HumanSession> humanSessions = new HashMap<String, HumanSession>();

	private XBoardEngineProcessManager engineProcessManager;

	private Connector connector;

	private Timer timer = new Timer();

	public SessionManager(XBoardEngineProcessManager engineProcessManager,
			Connector connector) {
		this.engineProcessManager = engineProcessManager;
		this.connector = connector;
		timer.schedule(new SessionCollectorTask(this), HALF_DAY, HALF_DAY);
	}

	public Session getSession(String id) {
		final Session session = xBoardSessions.get(id);
		if (session != null)
			return session;
		return humanSessions.get(id);
	}

	/**
	 * Creates human session
	 *
	 * @param sessionId
	 * @param publicSession
	 * @throws SessionIdExistsException
	 */
	public void createSession(final String userId, String sessionId,
			boolean publicSession) throws InvalidSessionIdException {
		if (humanSessions.get(sessionId) != null)
			throw new InvalidSessionIdException();
		final HumanSession session = new HumanSession(sessionId, this,
				connector, userId, publicSession);
		humanSessions.put(sessionId, session);
	}

	private long getNextXBoardSessionId() {
		final long id = nextXBoardSessionId++;
		if (nextXBoardSessionId == Long.MAX_VALUE)
			nextXBoardSessionId = Long.MIN_VALUE;
		return id;
	}

	/**
	 * Creates XBoard session
	 *
	 * @param sessionId
	 * @throws XBoardSessionsLimitExceededException
	 */
	public String createSession(final String userId)
			throws XBoardSessionsLimitExceededException {
		String sessionId;
		do {
			sessionId = "" + getNextXBoardSessionId();
		} while (getSession(sessionId) != null);
		final XBoardSession session = new XBoardSession(sessionId, this,
				engineProcessManager, connector, userId);
		xBoardSessions.put(sessionId, session);

		return sessionId;
	}

	/**
	 *
	 * Joins player to the specified session.
	 *
	 * @param sessionId
	 * @throws InvalidSessionIdException
	 *             when session doesn't exist or is already full
	 * @throws PublicSessionNotFoundException
	 */
	public void joinSession(String userId, String sessionId)
			throws InvalidSessionIdException, PublicSessionNotFoundException {
		HumanSession session = null;

		session = humanSessions.get(sessionId);
		if (session == null)
			throw new InvalidSessionIdException();
		session.join(userId);
		connector.sendMessage(session.getPrimaryPlayerId(), new StartMessage()
				.toString());
	}

	/**
	 *
	 * Joins player to any free public session.
	 *
	 * @throws PublicSessionNotFoundException
	 *
	 * @return session id of joined session
	 */
	public String joinSession(String userId)
			throws PublicSessionNotFoundException {
		HumanSession session = null;
		for (HumanSession humanSession : humanSessions.values()) {
			if (humanSession.isPublic()
					&& humanSession.getSecondaryPlayerId() == null) {
				session = humanSession;
				break;
			}
		}
		if (session == null) {
			throw new PublicSessionNotFoundException();
		}

		try {
			session.join(userId);
		} catch (InvalidSessionIdException e) {
			// ignore, won't happen becouse we have just checked that the
			// session is opened
		}
		connector.sendMessage(session.getPrimaryPlayerId(), new StartMessage()
				.toString());
		return session.getSessionId();
	}

	/**
	 *
	 * @param sessionId
	 * @param b
	 *            to send or not termination notification to the other client
	 */
	public synchronized void terminateSession(final String sessionId) {
		final Session session = getSession(sessionId);
		if (session != null) {
			session.terminate(true);
			removeSession(sessionId);
		}
	}

	private void removeSession(String sessionId) {
		humanSessions.remove(sessionId);
		xBoardSessions.remove(sessionId);
	}

	/**
	 * @return the humanSessions
	 */
	public Map<String, HumanSession> getHumanSessions() {
		return humanSessions;
	}

	/**
	 * @return the xBoardSessions
	 */
	public Map<String, XBoardSession> getXBoardSessions() {
		return xBoardSessions;
	}
}
